@@ -10,17 +10,23 @@ from .snapshot_ensemble import SnapshotEnsembleClassifier
 from .snapshot_ensemble import SnapshotEnsembleRegressor
 from .adversarial_training import AdversarialTrainingClassifier
 from .adversarial_training import AdversarialTrainingRegressor
+from .fast_geometric import FastGeometricClassifier
+from .fast_geometric import FastGeometricRegressor
 
 
-__all__ = ["FusionClassifier",
-           "FusionRegressor",
-           "VotingClassifier",
-           "VotingRegressor",
-           "BaggingClassifier",
-           "BaggingRegressor",
-           "GradientBoostingClassifier",
-           "GradientBoostingRegressor",
-           "SnapshotEnsembleClassifier",
-           "SnapshotEnsembleRegressor",
-           "AdversarialTrainingClassifier",
-           "AdversarialTrainingRegressor"]
+__all__ = [
+    "FusionClassifier",
+    "FusionRegressor",
+    "VotingClassifier",
+    "VotingRegressor",
+    "BaggingClassifier",
+    "BaggingRegressor",
+    "GradientBoostingClassifier",
+    "GradientBoostingRegressor",
+    "SnapshotEnsembleClassifier",
+    "SnapshotEnsembleRegressor",
+    "AdversarialTrainingClassifier",
+    "AdversarialTrainingRegressor",
+    "FastGeometricClassifier",
+    "FastGeometricRegressor",
+]
